/*
  "AD9850_51"
  library providing functions to control DDS modules based on AD9850 or AD9851 in serial mode (data pin == D7)
  (AD9850 is usually clocked by a 125MHz crystal whereas AD9851 uses a 30MHz crystal and internal 6x clock
   multiplier)
  V1.2 / Copyright (C) 2023, T.Rode (DL1DUZ)

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.

  Acknowledgements:
  Most algorithms were derived from Christophe Caiveaus library "Christophe Caiveau f4goj@free.fr" 
*/

#ifndef AD9850_51_h_
#define AD9850_51_h_

#if (ARDUINO >= 100)
#include <Arduino.h> 
#else
#include <WProgram.h> 
#endif

#include <stdint.h>

//################################################################################################################
//definitions
//################################################################################################################

class AD9850_51 {

public:
// constructor
  AD9850_51(const uint8_t w_clk_pn, const uint8_t fq_ud_pn, const uint8_t data_pn, const uint8_t reset_pn,
            const uint32_t clk_frequency, const uint8_t is_AD9851);
// destructor
  ~AD9850_51() { };

// resets DDS and enters parallel mode & power save mode (is automatically called by the constructor)
  void reset();
// sets deltaphase (phase-stepwidth) and initial phase; initial phase coded on 5 bits (value ranging from 0...32,
// each step representing an angle of 11.25 degree)
// if deltaphase is set to 0, DDS will shut down, reducing power consumption and dissipated heat to a minimum
  void setPhase(uint32_t deltaphase, uint8_t phase, const uint8_t shutdown);
// calculates the phase-value for a given frequency and DDS clock offset [Hz] caused by offset in crystal temperature
  unsigned long calculatePhaseValue(const uint32_t frequency, const int16_t clockOffset);
// calculates the frequency [Hz] for a give phase-value and clock offset [Hz] caused by offset in crystal temperature
  unsigned long calculateFrequency(const uint32_t phase, const int16_t clockOffset);
// support function that calculates the DDS clock frequency at nominal temperatur by comparing a set output to the
// actually measured one; e.g. if a 180MHz-nominal DDS is set to 10MHz and the measured output is 9.9997MHz
// "calculateClockFrequency(10000000, 9999700, 180000000)" returns 179994600Hz = 179.9946MHz; for good
// numeric accuracy the test frequency should be chosen as high as possible
  unsigned long calculateClockFrequency(const uint32_t f_set, const uint32_t f_real, const uint32_t clock);

private:
// AD9851 data pins
  const uint8_t W_CLK_PIN;
  const uint8_t FQ_UD_PIN;
  const uint8_t DATA_PIN;
  const uint8_t RESET_PIN;
// status-flag (if DDS is shut down, dissipated power is reduced to a minimum)
  uint8_t dds_is_down;
// actual DDS clock frequency [Hz] at nominal crystal temperature
  const uint32_t FC;
// Flag, indicating that chip is AD9851
  const uint8_t IS_9851;

  inline void ddsPulse(const uint8_t pin);
  inline void ddsShiftOut(const uint8_t data);
};

#endif // AD9850_51_h_